"use strict";


// Zeichengröße des canvas an die Anzeigegrösse anpassen (in style.css eingestellt)
function resizeCanvasToDisplaySize(canvas)
{
    // look up the size the canvas is being displayed
    const width = canvas.clientWidth;
    const height = canvas.clientHeight;

    // If it's resolution does not match change it
    if (canvas.width !== width || canvas.height !== height)
    {
        canvas.width = width;
        canvas.height = height;
        return true;
    }

    return false;
}


function setup_einheitskreis()
{
    var x0 = 0;
    var y0 = 0;

    var mouse = 
    {
        x: 0,
        y: 0,
        isDown: false
    };

    var frameRate  = 1 / 60;
    var frameDelay = frameRate * 1000;  // Intervallzeit zum Zeichnen in ms

    var canvas;
    var ctx;
    var width  = 0;
    var height = 0;

    var D = 10; // Durchmesser Markierung

    // Plot-Variablen
    var PLOT;
    var xx = [];
    var yy = [];

    var updateTime = 0;
    var updatePeriode = 20;    // Update des Plots alle 50ms

    var data = [
    {
        x: [],
        y: [],
        name: 'sin(Φ)',
        mode: 'markers',
        line:
        {
            width: 1,
            dash: 'solid'
        }
    },
    {
        x: [],
        y: [],
        name: '',
        mode: 'markers',
        marker:
        {
            size: 15
        },
        line:
        {
            width: 1,
            dash: 'solid'
        }
    }];

    var layout =
    {
        xaxis:
        {
            autotick: true,
            title: 'Winkel Φ/π',
            tickmode: 'linear',
            range: [0, 2]
        },
        yaxis:
        {
            autotick: true,
            title: 'y/R',
            range: [-1.5, +1.5]
        },
        margin:
        {
            l: 40,
            r: 30,
            b: 40,
            t: 30,
            pad: 0
        }
    };

    PLOT = document.getElementById('plot');
    Plotly.plot(PLOT, data, layout);

    canvas = document.getElementById('einheitskreis');
    ctx = canvas.getContext('2d');

    resizeCanvasToDisplaySize(canvas);

    width = canvas.width - 20;
    height = canvas.height - 20;
    x0 = canvas.width / 2;
    y0 = canvas.height / 2;

    // Maus-Events initialisieren
    canvas.onmousemove = getMousePosition;

    canvas.onmousedown = function(e)
    {
        if (e.which == 1)
        {
            getMousePosition(e);
            mouse.isDown = true;
        }
    };

    canvas.onmouseup = function(e)
    {
        if (e.which == 1)
        {
            mouse.isDown = false;
        }
    };

    setInterval(loop, frameDelay);
    
    
    function append(phi, y, reset)
    {
        if (reset)
        {
            xx = [];
            yy = [];
        }

        xx.push(phi);
        yy.push(y);
    }


    function update(phi, y)
    {
        var update =
        {
            x: [xx, [phi]],
            y: [yy, [y]]
        };

        Plotly.restyle(PLOT, update, [0, 1]);
    }


    function getMousePosition(e)
    {
        mouse.x = e.pageX - canvas.offsetLeft;
        mouse.y = e.pageY - canvas.offsetTop;
    }

    
    /*
     * loop() wird durch setInterval(loop, frameDelay) in setup() periodisch aufgerufen
     */
    function loop()
    {
        var phi;
        var xp, yp;
        var xm = mouse.x;
        var ym = mouse.y;

        xp = (mouse.x - x0) / (width / 2);
        yp = -(mouse.y - y0) / (width / 2);

        phi = Math.atan2(yp, xp);
        phi = (phi >= 0 ? phi : phi + 2 * Math.PI);

        xm = Math.cos(phi) * width / 2 + x0;
        ym = -Math.sin(phi) * width / 2 + y0;

        if (mouse.isDown)
        {
            append(phi / Math.PI, Math.sin(phi), false);
        }

        updateTime += frameDelay;
        if (updateTime >= updatePeriode)
        {
            updateTime = 0;
            update(phi / Math.PI, Math.sin(phi));
        }

        ctx.clearRect(0, 0, canvas.width, canvas.height);
        ctx.save();

        // x-Achse
        ctx.strokeStyle = 'black';
        ctx.lineCap = "round";
        ctx.beginPath();
        ctx.moveTo(x0 - width / 2, y0);                 
        ctx.lineTo(x0 + width / 2, y0);
        ctx.closePath();
        ctx.stroke();

        // y-Achse
        ctx.strokeStyle = 'red';                        
        ctx.lineCap = "round";
        ctx.beginPath();
        ctx.moveTo(x0, y0 - width / 2);
        ctx.lineTo(x0, y0 + width / 2);
        ctx.moveTo(x0, y0 - width / 2);  //Pfeilspitze
        ctx.lineTo(x0 - 5, y0 - width / 2 + 10);
        ctx.moveTo(x0, y0 - width / 2);
        ctx.lineTo(x0 + 5, y0 - width / 2 + 10);
        ctx.closePath();
        ctx.stroke();

        // Kreis
        ctx.strokeStyle = 'blue';
        ctx.beginPath();
        ctx.moveTo(x0 + width / 2, y0);
        ctx.arc(x0, y0, width / 2, 0, 2 * Math.PI);
        ctx.closePath();
        ctx.stroke();

        // Beschriftung
        ctx.fillStyle = 'red';
        ctx.font = '25px Georgia';
        ctx.fillText('y', x0, y0 - width / 2 - 20);
        ctx.fillText('0', x0 + 5, y0 - 10);   
        ctx.fillText('1', x0 + 5, y0 - width / 2 + 20);
        ctx.fillText('-1', x0 + 5, y0 + width / 2 - 2);

        // Markierung und Winkel
        ctx.fillStyle = 'red';
        ctx.beginPath();
        ctx.arc(xm, ym, D, 0, 2 * Math.PI);
        ctx.fill();
        ctx.closePath();
        ctx.stroke();

        ctx.strokeStyle = 'green';
        ctx.beginPath();
        ctx.moveTo(x0, y0);
        ctx.lineTo(xm, ym);
        ctx.closePath();
        ctx.stroke();

        ctx.strokeStyle = 'red';
        ctx.beginPath();
        ctx.moveTo(xm, y0);
        ctx.lineTo(xm, ym);
        ctx.closePath();
        ctx.stroke();

        ctx.fillStyle = 'tomato';
        ctx.beginPath();
        ctx.moveTo(x0, y0);
        ctx.arc(x0, y0, width / 4, 0, 2 * Math.PI - phi, true);
        ctx.fill();
        ctx.closePath();

        ctx.fillStyle = 'black';
        ctx.font = '25px Georgia';
        ctx.fillText('Φ', Math.cos(phi / 2) * width / 8 + x0, -Math.sin(phi / 2) * width / 8 + y0);

        ctx.fillStyle = 'red';
        ctx.font = '15px Georgia';
        ctx.fillText('y=sin(Φ)', Math.cos(phi) * width / 2 + 15 + x0, -Math.sin(phi) * width / 4 + y0);

        ctx.restore();
    }

}


function setup_riesenrad()
{
    var frameRate  = 1 / 20;

    var canvas;
    var ctx;
    var width  = 0;
    var height = 0;
    var x0, y0;

    var D = 10; // Durchmesser Markierung

    canvas = document.getElementById('riesenrad');
    ctx = canvas.getContext('2d');
    resizeCanvasToDisplaySize(canvas);

    width = canvas.width - 20;
    height = canvas.height - 20;
    x0 = canvas.width / 2;
    y0 = canvas.height / 2;


    // Winkel bei jedem Aufruf erhöhen, count_phi.phi dient als "static"-Variable
    function count_phi()
    {
        if (typeof count_phi.phi == 'undefined')
        {
            count_phi.phi = 3 / 4 * 2 * Math.PI;
        }

        count_phi.phi += Math.PI / 360;
        if (count_phi.phi >= 2 * Math.PI)
        {
            count_phi.phi = 0;
        }

        return count_phi.phi;
    }


    function loop()
    {
        var xm, ym;

        var phi = count_phi();

        xm = Math.cos(phi) * width / 2 + x0;
        ym = -Math.sin(phi) * width / 2 + y0;

        ctx.clearRect(0, 0, canvas.width, canvas.height);
        ctx.save();

        // x-Achse
        ctx.strokeStyle = 'black';
        ctx.lineCap = "round";
        ctx.beginPath();
        ctx.moveTo(x0 - width / 2, y0);
        ctx.lineTo(x0 + width / 2, y0);
        ctx.closePath();
        ctx.stroke();

        // y-Achse
        ctx.strokeStyle = 'red';
        ctx.lineCap = "round";
        ctx.beginPath();
        ctx.moveTo(x0, y0 - width / 2);
        ctx.lineTo(x0, y0 + width / 2);
        ctx.closePath();
        ctx.stroke();

        // Einheitskreis
        ctx.strokeStyle = 'blue';
        ctx.beginPath();
        ctx.moveTo(x0 + width / 2, y0);
        ctx.arc(x0, y0, width / 2, 0, 2 * Math.PI);
        ctx.closePath();
        ctx.stroke();

        // durchlaufener Winkel
        ctx.fillStyle = 'tomato';
        ctx.beginPath();
        ctx.moveTo(x0, y0);
        ctx.arc(x0, y0, width / 4, Math.PI / 2, Math.PI / 2 - (phi - 3 / 4 * 2 * Math.PI), true);
        ctx.fill();
        ctx.closePath();

        // Markierung
        ctx.fillStyle = 'red';
        ctx.beginPath();
        ctx.arc(xm, ym, D, 0, 2 * Math.PI);
        ctx.fill();
        ctx.closePath();

        // Radius
        ctx.strokeStyle = 'green';
        ctx.beginPath();
        ctx.moveTo(x0, y0);
        ctx.lineTo(xm, ym);
        ctx.closePath();
        ctx.stroke();

        // Gegenkathete (Sinus)
        ctx.strokeStyle = 'red';
        ctx.beginPath();
        ctx.moveTo(xm, y0);
        ctx.lineTo(xm, ym);
        ctx.closePath();        
        ctx.stroke();

        ctx.restore();
    }

    setInterval(loop, frameRate * 1000);
}
