"use strict";

(function()
{
    const updatePeriode = 300;  // alle XXms Plot updaten
    const ballR = 20;           // Ballradius
    const floor_height = 10;    // Höhe des Bodens
    const yscale = 100;         // Skalierung von Pixel in m
    const vyscale = 8;         // Skalierung von Pixel/s in m/s

    var startFlag = false;
    var startTime = 0;
    var updateTime = 0;
    var simulationStartTime;
    
    var t = [];
    var y = [];
    var vy = [];
    
    var isFalling = false;

    var canvas = document.getElementById('xworld');    
    var width = canvas.clientWidth;
    var height = canvas.clientHeight;
    
    //console.log(canvas);
    //resizeCanvasToDisplaySize(canvas);

    var engine = Matter.Engine.create({enableSleeping: true});
    var world = engine.world;
    var render = Matter.Render.create(
    {
        element: document.body,
        canvas: canvas,
        engine: engine,
        options:
        {
            width: width,
            height: height,
            //background: 'transparent',
            wireframes: false,
            showAngleIndicator: false
        }
    });

    var ball = Matter.Bodies.circle(width / 2, height / 2, ballR,
    {
        density: 0.04,
        friction: 0.01,
        frictionAir: 0.00001,
        restitution: 0.8,
        //restitution: 0.0,
        render:
        {
            fillStyle: '#F35e66',
            strokeStyle: 'black',
            lineWidth: 1
        }
    });

    Matter.World.add(world, ball);

    // Add a floor (Matter.js places a body’s origin at its centre!!)
    var floor = Matter.Bodies.rectangle(width / 2, height + floor_height / 2, width, floor_height,
    {
        isStatic: true, // An immovable object
        render:
        {
            //visible: false
        }
    });

    Matter.World.add(world, floor);

    // Make interactive
    var mouseConstraint = Matter.MouseConstraint.create(engine,
    {   //Create Constraint
        element: render.canvas,
        constraint:
        {
            render:
            {
                visible: true
            },
            stiffness:0.08
        }
    });

    Matter.World.add(world, mouseConstraint);
    
    Matter.Events.on(render, 'afterRender', function()
    {
        if (!startFlag && ball.isSleeping)
        {
            document.getElementById('status').innerHTML = 'Bitte heben Sie den Ball auf und lassen ihn fallen...';
            startFlag = true;
            startTime = engine.timing.timestamp;
            simulationStartTime = engine.timing.timestamp;
            updateTime = startTime;
        } else if (startFlag && ball.isSleeping) {
            append(true);
        }

        if (startFlag && !ball.isSleeping)
        {
            append(false);
            if (engine.timing.timestamp > updateTime + updatePeriode)
            {
                updateTime = engine.timing.timestamp;
                update();
            }
        }
    });

    var data = [
    {
        x: [],
        y: [],
        name: 'Position',
        mode: 'lines',
        line:
        {
            width: 1,
            dash: 'solid'
        }
    },
    {
        x: [],
        y: [],
        name: 'Geschwindigkeit',
        mode: 'lines',
        line:
        {
            width: 1,
            dash: 'dot'
        }
    }
    ]

    var layout =
    {    
        xaxis:
        {
            autotick: true,
            title: 't in s',
            tickmode: 'linear',
            range: [0, 6]
        },
        yaxis:
        {
            title: 'y(t), vy(t)',
            range: [-4, 4]
        }
    }

    var PLOT = document.getElementById('xplot');
    Plotly.plot(PLOT, data, layout);

    // Start the engine
    Matter.Engine.run(engine);
    Matter.Render.run(render);
    
    function append(reset)
    {
        if (reset)
        {
            t = [];
            y = [];
            vy = [];
            simulationStartTime = engine.timing.timestamp;
        }

        t.push((engine.timing.timestamp - simulationStartTime) / 1000.0);
        y.push((height - ballR - ball.position.y) / yscale);
        vy.push(-ball.velocity.y.toFixed(0) / vyscale);
        
        if (y[y.length - 1] < -1 && isFalling == false)
        {
            isFalling = true;
            console.log(y[y.length - 1]);
            var update =
            {
                //'xaxis.autorange': true,   // updates the xaxis range
                'yaxis.autorange': true
            };
            Plotly.relayout(PLOT, update);
        }
        
        if (t[t.length - 1] > 6 && typeof PLOT.layout.xaxis.autorange == 'undefined')
        {
            var update =
            {
                'xaxis.autorange': true
            };
            Plotly.relayout(PLOT, update);
        }
    }

    function update()
    {
        var update =
        {
            x: [t, t],
            y: [y, vy]
        }
        
        Plotly.restyle(PLOT, update, [0, 1]);
        console.log();
    }

    // Zeichengröße des canvas an die Anzeigegrösse anpassen (in style.css eingestellt)
    /*function resizeCanvasToDisplaySize(canvas)
    {
    // look up the size the canvas is being displayed
    const width = canvas.clientWidth;
    const height = canvas.clientHeight;
    
    console.log('client:', width, height);
    console.log('canvas:', canvas.width, canvas.height);

    // If it's resolution does not match change it
    if (canvas.width !== width || canvas.height !== height)
    {
        canvas.width = width;
        canvas.height = height;
        return true;
    }

    return false;
    }*/
})();
