// Quelle: https://wiki.selfhtml.org/wiki/JavaScript/Anwendung_und_Praxis/Funktionsplotter

"use strict";

// Globale Variablen, wegen Event-Aufruf der Funktion fkt_plotter()
var e_fkt = [], e_xmin, e_xmax, e_plotid, e_param, e_xaxis, e_yaxis;
var e_npt = 1000;

//function fkt_plotter(e_plotid, e_xmin, e_xmax, npt, ymin, ymax, e_fkt, e_axis, e_name)
function fkt_plotter()
{
	// Zahlen einlesen und bei Bedarf korrigieren
	function get_num(e)
	{
		var num = e.value;
		if (isNaN(num))
		{
			num = num.replace(/,/g, ".");
		}
		
		if (isNaN(num) || num.length == 0)
		{
			num = 0.0;
			e.value = num;
		}
		return parseFloat(num);
	}

	
    var funktionen = [];
    var daten = [];

    var xmin = get_num(e_xmin);
    var xmax = get_num(e_xmax);
        
    // Funktionen lesen
    for(var i = 0; i < e_fkt.length; i++)
    {
        funktionen[i] = e_fkt[i].value;
    }
    
    console.log(e_fkt, funktionen);

    // xmin und xmax lesen, prüfen und bei Bedarf korrigieren
    if (xmax == xmin)
    {
        xmin -= 0.5;
        xmax += 0.5;
        e_xmin.value = xmin;
        e_xmax.value = xmax;
    }
    else if (xmax < xmin)
    {
        var t = xmax;
        xmax = xmin;
        xmin = t;
        e_xmin.value = xmin;
        e_xmax.value = xmax;
    }

    var dx = (xmax - xmin) / e_npt;

    // Werte für Diagramm berechnen ...
    for(var i = 0; i < funktionen.length; i++)
    {
        try
        {
            // für nicht-leere Strings "/\S/" = whitespaces
            if (funktionen[i].length && funktionen[i].search(/\S/) != -1)
            {
                //var fun = new Function ("x", "with(Math){return(" + funktionen[i] + ")}");
                var fun = new Function (e_param, "with(Math){return(" + funktionen[i] + ")}");
                var xa = [];
                var ya = [];
                
                for (var x = xmin; x <= xmax; x += dx)
                {
                    xa.push(x);
                    ya.push(fun(x));
                }
                
                var trace =
                {
                    x: xa,
                    y: ya,
                    color: 'red',
                    mode: 'lines'
                };

                if (e_npt <= 50)
                {
                    trace.mode = 'lines+markers';
                }
                daten.push(trace);
            }
        }
        catch(e)
        {
            alert("Fehler in der Formel Nr " + (i + 1) + ":\n\n" + funktionen[i] + "\n\n" + e);
        }
    }

    // ... und plotten
    Plotly.newPlot(e_plotid, daten,
        {
            xaxis: {range: [xmin, xmax], title: e_xaxis},
            yaxis: {title: e_yaxis},
            showlegend: false,
            margin:
            {
                l: 50,
                r: 50,
                b: 50,
                t: 50,
                pad: 4
            }
        })

    Plotly.restyle(e_plotid, {'line.color': ['red', 'green', 'blue']});
}


function setup(funktionen, xminmax, option) {
	var i, j, e_jsfkt, e_vdeffkt;
	
	var current = 0;
	var cols = ["#ff0000", "#008000", "#0000ff"];

	var vdef_fkt =
	{
		AM:         "(1-0.3*sin(x/4))*sin(3*x)",
		PM:         "sin(3*x-2*sin(x/3))",
		"sin(x)/x": "sin(x)/x",
		sinh:       "(exp(x)-exp(-x))/2",
		cosh:       "(exp(x)+exp(-x))/2",
		tanh:       "(exp(x)-exp(-x))/(exp(x)+exp(-x))",
		Puls:       "(function(){var y=0;for(var ii=10;ii<30;ii++) y+=sin(ii*x);return y})(x)"
	};

	var js_fkt =
	{
		abs:    "abs(x)",
		acos:   "acos(x)",
		asin:   "asin(x)",
		atan:   "atan(x)",
		cos:    "cos(x)",
		exp:    "exp(x)",
		log:    "log(x)",
		pow:    "pow(x,2)",
		sin:    "sin(x)",
		sqrt:   "sqrt(x)",
		tan:    "tan(x)",
	};

    // Referenzen
	e_plotid = document.getElementById("funktionsplotter");
    e_xmin = document.getElementById("idxmin");
    e_xmax = document.getElementById("idxmax");
    e_jsfkt = document.querySelectorAll("#jsfkt button"); 
    e_vdeffkt = document.querySelectorAll("#vdeffkt button"); 

    for (i = 0; i < 3; i++)
    {
        e_fkt[i] = document.getElementById("fkt" + (i + 1));
        document.getElementById("f" + (i + 1)).style.color = cols[i];
    }

    // Die Eventhandler
    document.getElementById("plotbutton").addEventListener("click", fkt_plotter);

    // In das aktive Eingabefeld...
    document.getElementById("fkt1").addEventListener("click", function() {current = 0});
    document.getElementById("fkt2").addEventListener("click", function() {current = 1});
    document.getElementById("fkt3").addEventListener("click", function() {current = 2});

    // ...vordefinierte Funktionen oder...
    for (i = 0; i < e_vdeffkt.length; i++)
    {
        e_vdeffkt[i].addEventListener("click", function()
            {
                e_fkt[current].value = vdef_fkt[this.innerHTML]
            });
    }

    // ...Javascript-Funktionen eintragen
    for (i = 0; i < e_jsfkt.length; i++)
    {
        e_jsfkt[i].addEventListener("click", function()
            {
                e_fkt[current].value += "+" + js_fkt[this.innerHTML]
            });
    }

    // Falls Funktionen und Plotbereich übergeben wurde, diese übernehmen
        //console.log("Funktionen:", funktionen);
    console.log("xmin/xmax:", xminmax);

    if (arguments.length >= 1)
    {
		for (i = 0; i < funktionen.length; i++)
		{
			e_fkt[i].value = funktionen[i];
		}
    }
    
    if (arguments.length >= 2)
    {
        e_xmin.value = xminmax[0];
        e_xmax.value = xminmax[1];
    }

    e_param = "x";
	e_xaxis = e_param;
	e_yaxis = "y = f(x)";
    if (arguments.length >= 3)
    {
        if (option.hasOwnProperty("param"))
        {
            e_param = option.param;
        }
		
		if (option.hasOwnProperty("xaxis"))
        {
            e_xaxis = option.xaxis;
        }
		
		if (option.hasOwnProperty("yaxis"))
        {
            e_yaxis = option.yaxis;
        }
    }
    
    fkt_plotter();
}


var plot2 = function(plotid, xmin, xmax, npt, ymin, ymax, e_fkt, e_axis, e_name)
{
    var dx = (xmax - xmin) / npt;
    var daten = [];

    try
    {
        var funktion = e_fkt[0];
        var fun = new Function ("t", "with(Math){return(" + funktion + ")}");
        var xa = [];
        var ya = [];
        for (var t = xmin; t <= xmax; t += dx)
        {
            xa.push(t);
            ya.push(fun(t));
        }

        var trace =
        {
            x: xa,
            y: ya,
            name: e_name[0],
            mode: 'lines+markers'
        }
        daten.push(trace);

        funktion = e_fkt[1];
        fun = new Function ("t", "with(Math){return(" + funktion + ")}");
        xa = [];
        ya = [];
        for (t = xmin; t <= xmax; t += dx)
        {
            xa.push(t);
            ya.push(fun(t));
        }

        trace =
        {
            x: xa,
            y: ya,
            yaxis: 'y2',
            name: e_name[1],
            mode: 'lines+markers'
        }
        daten.push(trace);

    }
    catch(e)
    {
        alert("Fehler in der Formel!");
    }

    Plotly.newPlot(plotid, daten,
    {
        xaxis:
        {
            title: e_axis[0],
            range: [xmin, xmax]
        },
        yaxis:
        {
            title: e_axis[1],
            range: [ymin, ymax]
        },
        yaxis2:
        {
            titlefont: {color: '#ff7f0e'},
            tickfont: {color: '#ff7f0e'},
            anchor: 'x',
            overlaying: 'y',
            side: 'right',
            position: 0.2,
            range: [ymin, ymax],
            title: e_axis[2]
        },
        showlegend: false,
        margin:
        {
            l: 50,
            r: 50,
            b: 50,
            t: 50,
            pad: 4
        }
    })
}


var plot1 = function(plotid, xmin, xmax, npt, ymin, ymax, e_fkt, e_axis, e_name)
{    
    var dx = (xmax - xmin)/ npt;
    var daten = [];

    try
    {
        var funktion = e_fkt[0];
        var fun = new Function ("t", "with(Math){return(" + funktion + ")}");
        var xa = [];
        var ya = [];
        for (var t = xmin; t <= xmax; t += dx)
        {
            xa.push(t);
            ya.push(fun(t));
        }

        var trace =
        {
            x: xa,
            y: ya,
            name: e_name[0],
            mode: 'lines'
        }
        daten.push(trace);
    }
    catch(e)
    {
        alert("Fehler in der Formel!");
    }

    Plotly.newPlot(plotid, daten,
    {
        xaxis:
        {
            title: e_axis[0],
            range: [xmin, xmax]
        },
        yaxis:
        {
            title: e_axis[1],
            range: [ymin, ymax]
        },
        showlegend: false,
        margin:
        {
            l: 50,
            r: 50,
            b: 50,
            t: 50,
            pad: 4
        }
    })
}
