'use strict';

var ABCD = [1.0, 1.0, 0.0, 0.0];
var N = 32;

var x = Array.apply(null, {length: N});

var phi = x.map(function(value, index, array)
{
    return index * 2.0 * Math.PI / (array.length - 1)
});

var y = phi.map(function(value, index, array)
{
    return (ABCD[0] * Math.sin(ABCD[1] * value + ABCD[2]) + ABCD[3]);
}, ABCD);

var xc = Array.apply(null, {length: 10 * N});

var phic = xc.map(function(value, index, array)
{
    return index * 2.0 * Math.PI / (array.length - 1)
});

var yc = phic.map(function (value, index, array)
{
    return (ABCD[0] * Math.sin(ABCD[1] * value + ABCD[2]) + ABCD[3]);
}, ABCD);


function initInput()
{
    document.getElementById('inN').value = N;
    document.getElementById('inA').value = ABCD[0];
    document.getElementById('outA').value = ABCD[0];
    
    document.getElementById('inB').value = ABCD[1];
    document.getElementById('outB').value = ABCD[1];
    
    document.getElementById('inC').value = ABCD[2];
    document.getElementById('outC').value = ABCD[2];
    
    document.getElementById('inD').value = ABCD[3];
    document.getElementById('outD').value = ABCD[3];
}


var data = [
{
    x: phi,
    y: y,
    name: 'A ∙ sin(B ∙ φ + C) + D, N = ' + N,
    mode: 'lines+markers',
    line:
    {
        width: 1,
        dash: 'solid'
    }
},
{
    x: phic,
    y: yc,
    name: 'A ∙ sin(B ∙ φ + C) + D',
    mode: 'lines',

},
{
    x: phi,
    y: phi.map(function (value, index, array)
    {
        return Math.sin(value)
    }),
    name: 'sin(φ)',
    type: 'scatter',
    mode: 'lines',
    line:
    {
        width: 1,
        dash: 'dot'
    }
},
{
    x: phi,
    y: phi.map(function (value, index, array)
    {
        return Math.cos(value)
    }),
    name: 'cos(φ)',
    type: 'scatter',
    mode: 'lines',
    line:
    {
        width: 1,
        dash: 'dot'
    }
}
]

var layout =
{
    margin: {t: 50},
    xaxis:
    {
        autotick: false,
        title: 'φ in <i>rad</i>',
        tickmode: 'array',
        tickvals: [0.0, Math.PI / 2, Math.PI, 3 * Math.PI / 2, 2 * Math.PI],
        ticktext: ['0', 'π/2', 'π', '3 ∙ π / 2', '2 ∙ π']
    },
    yaxis:
    {
        range: [-3.0, 3.0],
        scaleanchor: 'x',
        title: 'y = <i>f</i>(φ)'
    }
}

var SIN = document.getElementById('sin');
Plotly.plot(SIN, data, layout);

function adjustphi(n)
{
    //console.log(n);
    if (n < 15 || n > 128)
    {
        n = N
        document.getElementById('inN').value = N;
    }
    else
    {
        N = n
    }
    
    var x = Array.apply(null, {length: N});
    phi = x.map(function(value, index, array)
    {
        return index * 2.0 * Math.PI / (array.length - 1)
    });
    
    var update =
    {
        name: ['A ∙ sin(B ∙ φ + C) + D, N = ' + N],
        x: [phi],
        y: [phi.map(function (value, index, array)
            {
                return ABCD[0] * Math.sin(ABCD[1] * value + ABCD[2]) + ABCD[3]
            })
        ]
    }
    Plotly.restyle('sin', update, [0]);
}


function adjustABCD(abcd)
{
    ABCD[0] = parseFloat(abcd[0]);
    ABCD[1] = parseFloat(abcd[1]);
    ABCD[2] = parseFloat(abcd[2]);
    ABCD[3] = parseFloat(abcd[3]);
    
    var update =
    {
        x: [phi, phic],
        y: [phi.map(function (value, index, array)
            {
                return ABCD[0] * Math.sin(ABCD[1] * value + ABCD[2]) + ABCD[3]
            }),
            phic.map(function (value, index, array)
            {
                return ABCD[0] * Math.sin(ABCD[1] * value + ABCD[2]) + ABCD[3]
            })
        ]
    }
    Plotly.restyle('sin', update, [0, 1]);
    
    switch (ABCD[2])
    {
        case 3.141:
            document.getElementById('labelC').innerHTML = ' = +π';
            break;
            
        case -3.141:
            document.getElementById('labelC').innerHTML = ' = -π';
            break;
            
        case 6.282:
            document.getElementById('labelC').innerHTML = ' = +2π';
            break;
            
        case -6.282:
            document.getElementById('labelC').innerHTML = ' = -2π';
            break;
            
        case 1.5705:
            document.getElementById('labelC').innerHTML = ' = +π/2';
            break;
            
        case -1.5705:
            document.getElementById('labelC').innerHTML = ' = -π/2';
            break;
            
        case 0.78525:
            document.getElementById('labelC').innerHTML = ' = +π/4';
            break;
            
        case -0.78525:
            document.getElementById('labelC').innerHTML = ' = -π/4';
            break;

        default:
            document.getElementById('labelC').innerHTML = '';
            break;
    }
}
