/*
* Quelle: http://burakkanber.com/blog/physics-in-javascript-car-suspension-part-1-spring-mass-damper/
*/

"use strict";

var k = -20;                // Federkonstante in kg / s^2
var b = -0.5;               // Dämpfung in kg / s

var y0 = 50;

var scale =  1;

var block = {y0: y0, y: 0, v: 0, mass: 0.8};                  // Block Position, Geschwindigkeit und Masse
var wall  = {y0: 30, A:20, y: 30,  ly: 30, v: 0, t: 0, frequency: 0}; // Wand bzw. Anregung

var Sy = 30;
var Sx = 30;    // Größe der Masse
var mouse = {x: 0, y: 0, isDown: false};

var frameRate  = 1 / 30;
var frameDelay = frameRate * 1000;  // Intervallzeit zum Zeichnen in ms

var canvas_spring;
var ctx_spring;
var width_spring  = 0;
var height_spring = 0;

var D = 20;	// Federdurchmesser
var g = 9.81 * 10;

// Plot-Variablen
var time = 0;
var tt = [];
var yy = [];
var isstatic = false;

var PLOT;

const updatePeriode = 0.100;  // alle 0.1s Plot updaten
var startFlag = true;
var startTime = 0;
var updateTime = 0;
// var vy = [];


/*
 * loop() wird durch setInterval(loop, frameDelay) in setup() periodisch aufgerufen
 */
var loop = function()
{   
    // Anregung
    wall.t += frameRate;
    wall.ly = wall.y;
    wall.y = wall.y0 + wall.A * Math.sin(2 * Math.PI * wall.frequency * wall.t);
    wall.v = (wall.y - wall.ly) / frameRate;
    
    time += frameRate;
    startTime += frameRate;

    // Integration der Bewegungsgleichung
    if (!mouse.isDown)
    {
        // Hook'sches Gesetz: F_spring = -k * dx
        var F_spring = k * ((block.y - wall.y) - block.y0);

        // Dämpfung: F_damper = -b * v
        var F_damper = b * (block.v - wall.v);

        // Beschleunigung aus Trägkeitskraft Fm = - m * a:
        //  (F_spring + F_damper) - m * a = 0
        var a = (F_spring + F_damper) / block.mass;

        // Erdbeschleunigung dazu:
        // a = a + g;

        // Integration mit Euler-Methode:
        //  Rechtecke der Breite dt und der Höhe f(t + dt) aufsummieren
        block.v += a * frameRate;
        block.y += block.v * frameRate;
    }


    // canvas löschen und Zeichenenstellungen auf dem Stack speichern
    ctx_spring.clearRect(0, 0, width_spring, height_spring);
    ctx_spring.save();

    // Feder zeichnen
    var dy = (block.y + block.y0 - wall.y)/10;
    for (var y = wall.y; y <= block.y + block.y0; y += dy)
    {
        ctx_spring.strokeStyle = 'blue';
        ctx_spring.beginPath();
        
        ctx_spring.moveTo(width_spring / 2 - D/2, y);
        ctx_spring.lineTo(width_spring / 2+D/2, y + dy/2);
        ctx_spring.lineTo(width_spring / 2 - D/2, y + dy);

        ctx_spring.stroke();
        ctx_spring.closePath();
    }

    // Wand
    ctx_spring.fillStyle = '#CCCCCC';
    ctx_spring.fillRect(0, 0, width_spring, wall.y);

    // Masse
    ctx_spring.beginPath();
    ctx_spring.fillStyle = 'tomato';
    ctx_spring.moveTo(width_spring - (width_spring / 2), block.y0 + block.y + Sy/2);
    ctx_spring.arc(width_spring - (width_spring / 2), block.y0 + block.y + Sy/2, Sx, 0,2*Math.PI);
    ctx_spring.fill();
    ctx_spring.closePath();
    //ctx_spring.stroke();
    
    ctx_spring.beginPath();
    ctx_spring.lineCap="round";

    ctx_spring.moveTo(20,2*(block.y0 + wall.y0)-Sx/2);      // x-Achse
    ctx_spring.lineTo(70,2*(block.y0 + wall.y0)-Sx/2);
    
    ctx_spring.moveTo(20,2*(block.y0 + wall.y0)-Sx/2+50);   // y-Achse
    ctx_spring.lineTo(20,2*(block.y0 + wall.y0)-Sx/2-50);
    
    ctx_spring.moveTo(20,2*(block.y0 + wall.y0)-Sx/2-50);   // linke Pfeilspitze
    ctx_spring.lineTo(15,2*(block.y0 + wall.y0)-Sx/2-40);
    
    ctx_spring.moveTo(20,2*(block.y0 + wall.y0)-Sx/2-50);   // rechte Pfeilspitze
    ctx_spring.lineTo(25,2*(block.y0 + wall.y0)-Sx/2-40);
    ctx_spring.stroke();
    ctx_spring.closePath();
    
    ctx_spring.fillStyle = 'blue';
    ctx_spring.font="14px Georgia";
    ctx_spring.fillText("s = 0",25,2*(block.y0 + wall.y0));
    
    ctx_spring.fillText("s",20,2*(block.y0 + wall.y0)-Sx/2-60);

    ctx_spring.restore();
	
    if (!isstatic)
    {
        // Display values
		var fn = Math.round(Math.sqrt(-k / block.mass) / 2 / Math.PI * 100) / 100;
        document.getElementById('fn').innerHTML = fn;
		
        if (startFlag || mouse.isDown)
        {
            append(false);
        }
    
        if (mouse.isDown)
        {
            startFlag = false;
        } else if (!mouse.isDown && !startFlag) {
            startTime = 0;
            startFlag = true;
            append(true);
        }

        if (time > updateTime + updatePeriode)
        {
            //console.log('update!');
            updateTime = time;
            update();
        }
    }
};


var setup = function()
{
    var data = [
    {
        x: [],
        y: [],
        name: 'Position',
        mode: 'lines',
        line:
        {
            width: 1,
            dash: 'solid'
        }
    }/*,
    {
        x: [],
        y: [],
        name: 'Geschwindigkeit',
        mode: 'lines',
        line:
        {
            width: 1,
            dash: 'solid'
        }
    }*/
    ]

    var layout =
    {
        xaxis:
        {
            autotick: true,
            title: 't in s',
            tickmode: 'linear',
			range: [0,30]
			
        },
        yaxis:
        {
            title: 'Position'
        }
    }

    PLOT = document.getElementById('plot');
    Plotly.plot(PLOT, data, layout);

    document.getElementById('k_label').innerHTML = k;
    document.getElementById('k_slider').value = Math.abs(k);

    document.getElementById('b_label').innerHTML = b;
    document.getElementById('b_slider').value = Math.abs(b);

    document.getElementById('f_label').innerHTML = wall.frequency;
    document.getElementById('f_slider').value = wall.frequency;

    document.getElementById('m_label').innerHTML = block.mass;
    document.getElementById('m_slider').value = block.mass;
    
    canvas_spring = document.getElementById('canvas_spring');
    ctx_spring = canvas_spring.getContext('2d');

    console.log('setup:', canvas_spring.width);
    console.log('setup:', canvas_spring.height);

    width_spring = canvas_spring.width;
    height_spring = canvas_spring.height;
	
	Sx = 20 + block.mass * 30 / (5 - 0.1);

    canvas_spring.onmousemove = getMousePosition;

    canvas_spring.onmousedown = function(e)
    {
        if (e.which == 1)
        {
            getMousePosition(e);
            mouse.isDown = true;
            block.y = mouse.y - Sy / 2 - block.y0;
        }
    };

    canvas_spring.onmouseup = function(e)
    {
        if (e.which == 1)
        {
            mouse.isDown = false;
        }
    };

    document.getElementById('k_slider').onchange = function()
    {
        if (this.value == 0) this.value = -1;
        this.innerHTML = this.value;
        k = -1 * parseInt(this.value);
        block.y0 = y0 - block.mass * g / k;
        document.getElementById('k_label').innerHTML = k;
    };

    document.getElementById('b_slider').onchange = function()
    {
        this.innerHTML = this.value;
        b = -1 * parseFloat(this.value);
        document.getElementById('b_label').innerHTML = b;
    };

    document.getElementById('f_slider').onchange = function()
    {
        this.innerHTML = this.value;
        wall.frequency = parseFloat(this.value);
        document.getElementById('f_label').innerHTML = wall.frequency;
    };

    document.getElementById('m_slider').onchange = function()
    {
        this.innerHTML = this.value;
        block.mass = parseFloat(this.value);
        block.y0 = y0 - block.mass * g / k;
		Sx = 20 + block.mass * 30 / (5 - 0.1);
        document.getElementById('m_label').innerHTML = block.mass;
    };

    setInterval(loop, frameDelay);
};

function append(reset)
{
    if (reset)
    {
        tt = [];
        yy = [];
    }
    tt.push(startTime);
    yy.push(-(block.y - wall.y - block.y0));
	
	if (tt[tt.length - 1] > 30 && typeof PLOT.layout.xaxis.autorange == 'undefined')
	{
		var update =
		{
			'xaxis.autorange': true
		};
		Plotly.relayout(PLOT, update);
	}
}

function update()
{
    var update =
    {
        x: [tt],
        y: [yy]
    }
    Plotly.restyle(PLOT, update, [0]);
}

var getMousePosition = function(e)
{
    mouse.x = e.pageX - canvas_spring.offsetLeft;
    mouse.y = e.pageY - canvas_spring.offsetTop;
    if (mouse.isDown)
    {
        block.y = mouse.y - Sy / 2 - block.y0;
    }
};


var static_setup = function()
{
    canvas_spring = document.getElementById('canvas_spring');
    ctx_spring = canvas_spring.getContext('2d');
    
    document.getElementById('pausebutton').onclick = function()
    {
        b = -2.0;
    };
    
    isstatic = true;

    //console.log(canvas_spring.width);
    //console.log(canvas_spring.height);
    b = 0.0;
    scale = 1;
    
    block = {y0: y0 / scale, y: 0, v: 0, mass: 0.5};                  // Block Position, Geschwindigkeit und Masse
    wall  = {y0: 30 / scale, A: 20 / scale, y: 30 / scale,  ly: 30 / scale, v: 0, t: 0, frequency: 0}; // Wand bzw. Anregung
    Sy = Sy / scale;
    Sx = Sx / scale;    // Größe der Masse
    D = D / scale;
    
    width_spring = canvas_spring.width;
    height_spring = canvas_spring.height;

    setInterval(loop, frameDelay);
};
